@@ -9,11 +9,16 @@ use IO::AIO;
 BEGIN {
    use XSLoader;
 
-   our $VERSION = '1.61';
+   our $VERSION = '1.7';
 
    XSLoader::load "Gtk2::CV", $VERSION;
 }
 
+magic_buffer ""; # preload magic tables
+
+our $MPLAYER = $ENV{CV_MPLAYER} || "mpv";
+our $MPLAYER_IS_MPV = $MPLAYER =~ /mpv/; # sorry if this fails for you
+
 our $FAST_TMP = -w "/run/shm" ? "/run/shm"
               : -w "/dev/shm" ? "/dev/shm"
               :                 "/tmp";
@@ -77,5 +82,25 @@ sub dealpha($) {
    &dealpha_compose
 }
 
+sub load_webp($;$$$) {
+   my ($path, $thumbnail, $iw, $ih) = @_;
+
+   open my $fh, "<:raw", $path
+      or die "$path: $!\n";
+   IO::AIO::mmap my $data, -s $fh, IO::AIO::PROT_READ, IO::AIO::MAP_SHARED, $fh
+      or die "$path: $!\n";
+   decode_webp $data, $thumbnail, $iw, $ih
+}
+
+sub load_jpeg($;$$$) {
+   my ($path, $thumbnail, $iw, $ih) = @_;
+
+   open my $fh, "<:raw", $path
+      or die "$path: $!\n";
+   IO::AIO::mmap my $data, -s $fh, IO::AIO::PROT_READ, IO::AIO::MAP_SHARED, $fh
+      or die "$path: $!\n";
+   decode_jpeg $data, $thumbnail, $iw, $ih
+}
+
 1;
 
